{*
 *
 *  This file is part of Quizzer.
 *
 *  Copyright (C) 2008 Martijn van der Horst <M.G.v.d.Horst@gmail.com>
 *
 *  Quizzer is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *}


unit Unit1; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, StdCtrls, synaser,
  ExtCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    Status: TLabel;
    OrderList: TLabel;
    Main: TLabel;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: char);
    procedure FormResize(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { private declarations }
    serial: TBlockSerial;
    order: string;
    currentletter : integer;
    procedure OnSerialStatus(Sender: TObject; Reason: THookSerialReason;const Value: string);
    procedure checkSerial;
    procedure addLetter(letter:char);
    procedure clearOrder;
    procedure moveLetter(direction:integer);
    procedure updateScreen;
  public
    { public declarations }
  end; 

var
  Form1: TForm1; 

implementation

{ TForm1 }

procedure TForm1.OnSerialStatus(Sender: TObject; Reason: THookSerialReason;const Value: string);
begin
  checkSerial;
  Status.Caption:=serial.GetVersion + value  + 'status: ' + InttoStr(serial.ModemStatus);
end;

procedure TForm1.checkSerial;
begin
  if (serial.Carrier) then addLetter('A');
  if (serial.DSR) then addLetter('B');
  if (serial.CTS) then addLetter('C');
  if (serial.Ring) then addLetter('D');
  Status.Caption:=serial.GetVersion + 'status: ' + InttoStr(serial.ModemStatus);
end;

procedure TForm1.addLetter(letter:char);
var found : boolean;
    i : integer;
begin
  // Check if the letter is already in the order
  found:=false;
  for i:=1 to length(order) do found:=found or (order[i] = letter);

  // If it is not in the order, then we can add it
  if (not found) then begin
    order := order + letter;
    if (currentletter < 0) then currentletter:=0;
    updateScreen;
  end;
end;

procedure TForm1.clearOrder;
begin;
  order:='';
  currentletter := -1;
  moveLetter(0);
end;

procedure TForm1.moveLetter(direction:integer);
begin
  currentletter := currentletter + direction;
  if (currentletter < 0) then currentletter := 0;
  if (currentletter >= length(order)) then currentletter := length(order)-1;
  updateScreen;
end;

procedure TForm1.updateScreen;
var tmp:string;
begin
  if (currentletter < 0) then begin
    Main.Caption:='';
    OrderList.Caption:='';
  end else begin
    Main.Caption := order[currentletter+1];
    tmp:=order;
    tmp[currentletter+1]:=' ';
    OrderList.Caption:=tmp;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  serial := TBlockSerial.Create;
  serial.OnStatus:= @OnSerialStatus;
  serial.Purge;
  serial.Connect('COM1');
  serial.DTR:=true;
  Timer1.Enabled:=true;
  clearOrder;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  serial.Free;
end;

procedure TForm1.FormKeyPress(Sender: TObject; var Key: char);
begin
  if Key=' ' then clearOrder
  else if ord(Key) = 44 then moveLetter(-1)
  else if ord(Key) = 46 then moveLetter(+1)
  else addLetter(Key);
  Status.Caption:=serial.GetVersion + 'status: ' + InttoStr(serial.ModemStatus);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  Main.Font.Size:=(Self.Height*2) div 3;
  OrderList.Font.Size:=(Self.Height) div 10;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  checkSerial;
end;

initialization
  {$I unit1.lrs}

end.

